/**
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (c) 2010, Aleksei Valikov, Highsource.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Aleksei Valikov nor the
 *       names of contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ALEKSEI VALIKOV BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.hisrc.jsonix.compiler;

import org.apache.commons.lang.Validate;
import org.hisrc.jscm.codemodel.expression.JSMemberExpression;
import org.jvnet.jaxb2_commons.xml.bind.model.MBuiltinLeafInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MClassInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MEnumLeafInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MList;
import org.jvnet.jaxb2_commons.xml.bind.model.MPackagedTypeInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MTypeInfoVisitor;
import org.jvnet.jaxb2_commons.xml.bind.model.MWildcardTypeInfo;

final class CreateTypeInfoDeclarationVisitor<T, C extends T> implements
		MTypeInfoVisitor<T, C, JSMemberExpression> {

	private final JsonixCode code;

	private final JsonixCompiler<T, C> jsonixCompiler;

	CreateTypeInfoDeclarationVisitor(JsonixCompiler<T, C> jsonixCompiler,
			JsonixCode code) {
		Validate.notNull(jsonixCompiler);
		Validate.notNull(code);
		this.code = code;
		this.jsonixCompiler = jsonixCompiler;
	}

	private JSMemberExpression createTypeInfoDeclaration(
			MPackagedTypeInfo<T, C> info, JSMemberExpression typeInfoClass) {
		final JsonixModule module = this.jsonixCompiler.getModule(info);

		final JSMemberExpression typeInfoDeclaration = module.space.p(info
				.getLocalName());

		// final MemberNew typeInfoExpression =
		// typeInfoClass.instantiate().args(
		//
		// this.code.codeModel.object().append(
		// "name",
		// this.code.codeModel.string(module.spaceName + "."
		// + info.getLocalName())));

		// module.spaceObject.append(info
		// .getLocalName(), typeInfoExpression);

		return typeInfoDeclaration;

	}

	public JSMemberExpression visitEnumLeafInfo(MEnumLeafInfo<T, C> info) {
		// TODO Enums not supported yet, see JSNX-???
		// return null;
		// return createTypeInfoDeclaration(info,
		// this.code.$JsonixModelEnumLeafInfo);
		return this.jsonixCompiler.getTypeInfoDeclaration(info
				.getBaseTypeInfo());
	}

	public JSMemberExpression visitClassInfo(MClassInfo<T, C> info) {
		return createTypeInfoDeclaration(info, this.code.$JsonixModelClassInfo);
	}

	public JSMemberExpression visitList(MList<T, C> info) {
		return this.code.$JsonixSchemaXSDList.instantiate().args(
				this.jsonixCompiler.getTypeInfoDeclaration(info
						.getItemTypeInfo()));
	}

	public JSMemberExpression visitBuiltinLeafInfo(MBuiltinLeafInfo<T, C> info) {
		return null;
	}

	public JSMemberExpression visitWildcardTypeInfo(MWildcardTypeInfo<T, C> info) {
		return null;
	}
}